/******************************************************************************
 * @file     main.c
 * @version  V1.00
 * $Revision: 1 $
 * $Date: 2018/10/18 10:29 $
 * @brief    Main code on APROM
 * @note
 * Copyright (C) 2018 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/

#include <stdio.h>
#include "NM1230.h"

extern void CodeOnSPROM00(void);
extern int CodeOnSPROM1(int cnt);
extern int CodeOnSPROM2(void);
extern void CodeOnSPROM02(void);

int cnt = 0;
int abc = 0;
int def = 0;

int YYYYXXX  = 100;

void SYS_Init(void)
{
    /* Unlock protected registers */
    SYS_UnlockReg();

    CLK_SetCoreClock(FREQ_48MHZ);

    /* Enable USCI0 IP clock */
    CLK_EnableModuleClock(USCI0_MODULE);

    /* USCI-Uart0-GPD5(TX) + GPD6(RX) */
    /* Set GPD multi-function pins for USCI UART0 GPD5(TX) and GPD6(RX) */	
    SYS->GPD_MFP = SYS->GPD_MFP & ~(SYS_GPD_MFP_PD5MFP_Msk | SYS_GPD_MFP_PD6MFP_Msk) | (SYS_GPD_MFP_PD5_UART0_TXD | SYS_GPD_MFP_PD6_UART0_RXD);	
	
    /* Set GPD5 as output mode and GPD6 as Input mode */
    PD->MODE = PD->MODE & ~(GPIO_MODE_MODE5_Msk | GPIO_MODE_MODE6_Msk) | (GPIO_MODE_OUTPUT << GPIO_MODE_MODE5_Pos);
	
    /* Lock protected registers */
    SYS_LockReg();	
}

int main (void)
{

	SYS_UnlockReg();
	SYS_Init();

	
	SYS->IPRST1 |= SYS_IPRST1_TMR1RST_Msk;
  SYS->IPRST1 &= ~SYS_IPRST1_TMR1RST_Msk;
	
	CLK->CLKSEL1 |= 0x7700;
	CLK->APBCLK |= 0xC;
	
	TIMER1->CMP = 120000; //0.5  second
	TIMER1->CTL = 0x680000C8;
	
	GPIO_SetMode(PA,BIT0,GPIO_MODE_INPUT);
	GPIO_SetMode(PA,BIT1,GPIO_MODE_OUTPUT);	
	GPIO_SetMode(PA,BIT2,GPIO_MODE_OUTPUT);
	GPIO_SetMode(PA,BIT3,GPIO_MODE_OUTPUT);
	GPIO_SetMode(PA,BIT4,GPIO_MODE_OUTPUT);
	GPIO_SetMode(PA,BIT5,GPIO_MODE_OUTPUT);	
	GPIO_SetMode(PA,BIT6,GPIO_MODE_OUTPUT);
	GPIO_SetMode(PA,BIT7,GPIO_MODE_OUTPUT);
	GPIO_SetMode(PD,BIT7,GPIO_MODE_OUTPUT);
	GPIO_SetMode(PC,BIT1,GPIO_MODE_OUTPUT);
	NVIC_EnableIRQ(TMR1_IRQn);
	
	while(1){};
}

void TMR1_IRQHandler(void){
	TIMER1->INTSTS |= 0x1;
	CodeOnSPROM00();							//LED on and off
	abc = CodeOnSPROM1(cnt);
	def = CodeOnSPROM2();
}


/*** (C) COPYRIGHT 2018 Nuvoton Technology Corp. ***/
